@@ -92,6 +92,41 @@ static Bounds computeBounds(const Mesh& mesh, cgltf_attribute_type type)
 	return b;
 }
 
+static float computeUvArea(const Mesh& mesh)
+{
+	if (mesh.indices.empty() || mesh.type != cgltf_primitive_type_triangles)
+		return 0.f;
+
+	float result = 0.f;
+
+	for (size_t j = 0; j < mesh.streams.size(); ++j)
+	{
+		const Stream& s = mesh.streams[j];
+
+		if (s.type != cgltf_attribute_type_texcoord)
+			continue;
+
+		float uvarea = 0.f;
+
+		for (size_t i = 0; i < mesh.indices.size(); i += 3)
+		{
+			unsigned int a = mesh.indices[i + 0];
+			unsigned int b = mesh.indices[i + 1];
+			unsigned int c = mesh.indices[i + 2];
+
+			const Attr& va = s.data[a];
+			const Attr& vb = s.data[b];
+			const Attr& vc = s.data[c];
+
+			uvarea += fabsf((vb.f[0] - va.f[0]) * (vc.f[1] - va.f[1]) - (vc.f[0] - va.f[0]) * (vb.f[1] - va.f[1]));
+		}
+
+		result = std::max(result, uvarea / float(mesh.indices.size() / 3));
+	}
+
+	return result;
+}
+
 QuantizationPosition prepareQuantizationPosition(const std::vector<Mesh>& meshes, const Settings& settings)
 {
 	QuantizationPosition result = {};
@@ -191,6 +226,38 @@ void prepareQuantizationTexture(cgltf_data* data, std::vector<QuantizationTextur
 		bounds[indices[i]].merge(mb);
 	}
 
+	// detect potential precision issues and warn about them
+	if (settings.quantize && !settings.tex_float)
+	{
+		float max_rel_error = 0;
+
+		for (size_t i = 0; i < meshes.size(); ++i)
+		{
+			const Mesh& mesh = meshes[i];
+
+			if (!mesh.material && mesh.variants.empty())
+				continue;
+
+			const Bounds& b = bounds[indices[i]];
+			if (!b.isValid())
+				continue;
+
+			float scale = std::max(b.max.f[0] - b.min.f[0], b.max.f[1] - b.min.f[1]);
+			float error = scale * 0.5f / (1 << (settings.tex_bits - 1));
+
+			if (error < 1e-3f)
+				continue;
+
+			float uvarea = computeUvArea(mesh);
+			float rel_error = uvarea > 0 ? error / sqrtf(uvarea) : 0.f;
+
+			max_rel_error = std::max(max_rel_error, rel_error);
+		}
+
+		if (max_rel_error > 1e-1f)
+			fprintf(stderr, "Warning: texture coordinate data has significant error (%.0f%%); consider using floating-point quantization (-vtf) or more bits (-vt N)\n", max_rel_error * 100);
+	}
+
 	// update all material data using canonical bounds
 	for (size_t i = 0; i < result.size(); ++i)
 	{
@@ -313,6 +380,45 @@ static void encodeSnorm(void* destination, size_t count, size_t stride, int bits
 	}
 }
 
+static int quantizeColor(float v, int bytebits, int bits)
+{
+	int result = meshopt_quantizeUnorm(v, bytebits);
+
+	// replicate the top bit into the low significant bits
+	const int mask = (1 << (bytebits - bits)) - 1;
+
+	return (result & ~mask) | (mask & -(result >> (bytebits - 1)));
+}
+
+static void encodeColor(void* destination, size_t count, size_t stride, int bits, const float* data)
+{
+	assert(stride == 4 || stride == 8);
+	assert(bits >= 2 && bits <= 16);
+
+	unsigned char* d8 = static_cast<unsigned char*>(destination);
+	unsigned short* d16 = static_cast<unsigned short*>(destination);
+
+	for (size_t i = 0; i < count; ++i)
+	{
+		const float* c = &data[i * 4];
+
+		if (stride == 4)
+		{
+			d8[i * 4 + 0] = uint8_t(quantizeColor(c[0], 8, bits));
+			d8[i * 4 + 1] = uint8_t(quantizeColor(c[1], 8, bits));
+			d8[i * 4 + 2] = uint8_t(quantizeColor(c[2], 8, bits));
+			d8[i * 4 + 3] = uint8_t(quantizeColor(c[3], 8, bits));
+		}
+		else
+		{
+			d16[i * 4 + 0] = uint16_t(quantizeColor(c[0], 16, bits));
+			d16[i * 4 + 1] = uint16_t(quantizeColor(c[1], 16, bits));
+			d16[i * 4 + 2] = uint16_t(quantizeColor(c[2], 16, bits));
+			d16[i * 4 + 3] = uint16_t(quantizeColor(c[3], 16, bits));
+		}
+	}
+}
+
 static StreamFormat writeVertexStreamRaw(std::string& bin, const Stream& stream, cgltf_type type, size_t components)
 {
 	assert(components >= 1 && components <= 4);
@@ -328,11 +434,11 @@ static StreamFormat writeVertexStreamRaw(std::string& bin, const Stream& stream,
 	return format;
 }
 
-static StreamFormat writeVertexStreamFloat(std::string& bin, const Stream& stream, cgltf_type type, int components, const Settings& settings, int bits, meshopt_EncodeExpMode mode)
+static StreamFormat writeVertexStreamFloat(std::string& bin, const Stream& stream, cgltf_type type, int components, bool expf, int bits, meshopt_EncodeExpMode mode)
 {
 	assert(components >= 1 && components <= 4);
 
-	StreamFormat::Filter filter = settings.compress ? StreamFormat::Filter_Exp : StreamFormat::Filter_None;
+	StreamFormat::Filter filter = expf ? StreamFormat::Filter_Exp : StreamFormat::Filter_None;
 
 	if (filter == StreamFormat::Filter_Exp)
 	{
@@ -360,17 +466,7 @@ static StreamFormat writeVertexStreamFloat(std::string& bin, const Stream& strea
 	return format;
 }
 
-static int quantizeColor(float v, int bytebits, int bits)
-{
-	int result = meshopt_quantizeUnorm(v, bytebits);
-
-	// replicate the top bit into the low significant bits
-	const int mask = (1 << (bytebits - bits)) - 1;
-
-	return (result & ~mask) | (mask & -(result >> (bytebits - 1)));
-}
-
-StreamFormat writeVertexStream(std::string& bin, const Stream& stream, const QuantizationPosition& qp, const QuantizationTexture& qt, const Settings& settings)
+StreamFormat writeVertexStream(std::string& bin, const Stream& stream, const QuantizationPosition& qp, const QuantizationTexture& qt, const Settings& settings, bool filters)
 {
 	if (stream.type == cgltf_attribute_type_position)
 	{
@@ -378,7 +474,8 @@ StreamFormat writeVertexStream(std::string& bin, const Stream& stream, const Qua
 			return writeVertexStreamRaw(bin, stream, cgltf_type_vec3, 3);
 
 		if (settings.pos_float)
-			return writeVertexStreamFloat(bin, stream, cgltf_type_vec3, 3, settings, qp.bits, settings.compressmore ? meshopt_EncodeExpSharedComponent : meshopt_EncodeExpSeparate);
+			return writeVertexStreamFloat(bin, stream, cgltf_type_vec3, 3, settings.compress && filters, qp.bits,
+			    settings.compressmore ? meshopt_EncodeExpSharedComponent : meshopt_EncodeExpSeparate);
 
 		if (stream.target == 0)
 		{
@@ -458,7 +555,7 @@ StreamFormat writeVertexStream(std::string& bin, const Stream& stream, const Qua
 		// expand the encoded range to ensure it covers [0..1) interval
 		// this can slightly reduce precision but we should not need more precision inside 0..1, and this significantly improves compressed size when using encodeExpOne
 		if (settings.tex_float)
-			return writeVertexStreamFloat(bin, stream, cgltf_type_vec2, 2, settings, qt.bits, settings.compressmore ? meshopt_EncodeExpSharedComponent : meshopt_EncodeExpClamped);
+			return writeVertexStreamFloat(bin, stream, cgltf_type_vec2, 2, settings.compress && filters, qt.bits, meshopt_EncodeExpClamped);
 
 		float uv_rscale[2] = {
 		    qt.scale[0] == 0.f ? 0.f : 1.f / qt.scale[0],
@@ -486,9 +583,10 @@ StreamFormat writeVertexStream(std::string& bin, const Stream& stream, const Qua
 
 		// expand the encoded range to ensure it covers [0..1) interval
 		if (settings.nrm_float)
-			return writeVertexStreamFloat(bin, stream, cgltf_type_vec3, 3, settings, settings.nrm_bits, settings.compressmore || stream.target ? meshopt_EncodeExpSharedComponent : meshopt_EncodeExpClamped);
+			return writeVertexStreamFloat(bin, stream, cgltf_type_vec3, 3, settings.compress && filters, settings.nrm_bits,
+			    (settings.compressmore || stream.target) ? meshopt_EncodeExpSharedComponent : meshopt_EncodeExpClamped);
 
-		bool oct = settings.compressmore && stream.target == 0;
+		bool oct = filters && settings.compressmore && stream.target == 0;
 		int bits = settings.nrm_bits;
 
 		StreamFormat::Filter filter = oct ? StreamFormat::Filter_Oct : StreamFormat::Filter_None;
@@ -511,7 +609,7 @@ StreamFormat writeVertexStream(std::string& bin, const Stream& stream, const Qua
 		if (!settings.quantize)
 			return writeVertexStreamRaw(bin, stream, cgltf_type_vec4, 4);
 
-		bool oct = settings.compressmore && stream.target == 0;
+		bool oct = filters && settings.compressmore && stream.target == 0;
 		int bits = (settings.nrm_bits > 8) ? 8 : settings.nrm_bits;
 
 		StreamFormat::Filter filter = oct ? StreamFormat::Filter_Oct : StreamFormat::Filter_None;
@@ -531,40 +629,28 @@ StreamFormat writeVertexStream(std::string& bin, const Stream& stream, const Qua
 	}
 	else if (stream.type == cgltf_attribute_type_color)
 	{
+		bool col = filters && settings.compresskhr && settings.compressmore;
 		int bits = settings.col_bits;
 
-		for (size_t i = 0; i < stream.data.size(); ++i)
-		{
-			const Attr& a = stream.data[i];
+		StreamFormat::Filter filter = col ? StreamFormat::Filter_Color : StreamFormat::Filter_None;
 
-			if (bits > 8)
-			{
-				uint16_t v[4] = {
-				    uint16_t(quantizeColor(a.f[0], 16, bits)),
-				    uint16_t(quantizeColor(a.f[1], 16, bits)),
-				    uint16_t(quantizeColor(a.f[2], 16, bits)),
-				    uint16_t(quantizeColor(a.f[3], 16, bits))};
-				bin.append(reinterpret_cast<const char*>(v), sizeof(v));
-			}
-			else
-			{
-				uint8_t v[4] = {
-				    uint8_t(quantizeColor(a.f[0], 8, bits)),
-				    uint8_t(quantizeColor(a.f[1], 8, bits)),
-				    uint8_t(quantizeColor(a.f[2], 8, bits)),
-				    uint8_t(quantizeColor(a.f[3], 8, bits))};
-				bin.append(reinterpret_cast<const char*>(v), sizeof(v));
-			}
-		}
+		size_t offset = bin.size();
+		size_t stride = bits > 8 ? 8 : 4;
+		bin.resize(bin.size() + stream.data.size() * stride);
+
+		if (col)
+			meshopt_encodeFilterColor(&bin[offset], stream.data.size(), stride, bits, stream.data[0].f);
+		else
+			encodeColor(&bin[offset], stream.data.size(), stride, bits, stream.data[0].f);
 
 		if (bits > 8)
 		{
-			StreamFormat format = {cgltf_type_vec4, cgltf_component_type_r_16u, true, 8};
+			StreamFormat format = {cgltf_type_vec4, cgltf_component_type_r_16u, true, 8, filter};
 			return format;
 		}
 		else
 		{
-			StreamFormat format = {cgltf_type_vec4, cgltf_component_type_r_8u, true, 4};
+			StreamFormat format = {cgltf_type_vec4, cgltf_component_type_r_8u, true, 4, filter};
 			return format;
 		}
 	}
@@ -639,7 +725,7 @@ StreamFormat writeVertexStream(std::string& bin, const Stream& stream, const Qua
 	else if (stream.type == cgltf_attribute_type_custom)
 	{
 		// note: _custom is equivalent to _ID, as such the data contains scalar integers
-		if (!settings.compressmore)
+		if (!settings.compressmore || !filters)
 			return writeVertexStreamRaw(bin, stream, cgltf_type_scalar, 1);
 
 		unsigned int maxv = 0;
@@ -780,12 +866,12 @@ StreamFormat writeKeyframeStream(std::string& bin, cgltf_animation_path_type typ
 	}
 }
 
-void compressVertexStream(std::string& bin, const std::string& data, size_t count, size_t stride)
+void compressVertexStream(std::string& bin, const std::string& data, size_t count, size_t stride, int level)
 {
 	assert(data.size() == count * stride);
 
 	std::vector<unsigned char> compressed(meshopt_encodeVertexBufferBound(count, stride));
-	size_t size = meshopt_encodeVertexBuffer(&compressed[0], compressed.size(), data.c_str(), count, stride);
+	size_t size = meshopt_encodeVertexBufferLevel(&compressed[0], compressed.size(), data.c_str(), count, stride, level, level > 0);
 
 	bin.append(reinterpret_cast<const char*>(&compressed[0]), size);
 }
